#include "BinFile.h"

void BinFile::make_map(std::map<int, File>& map) {
    File f_info;
    int idx = 1;
    std::ifstream bin_file(fname, std::ios::binary);
    while (bin_file.read(reinterpret_cast<char*>(&f_info), sizeof(File))) {
        map.insert(std::make_pair(idx, f_info));
        ++idx;
    }
}

void BinFile::add_note() {
    std::ofstream bin_file(fname, std::ios::binary | std::ios::app);
    if (!bin_file.is_open()) {
        std::cerr << "Error: cant open output file " << fname << std::endl;
        exit(EXIT_FAILURE);
    }
    bool flag = 0;
    File f_info;
    std::cout << "Input the file information" << std::endl;
    while (!flag) {
        std::cin >> f_info;
        bin_file.write(reinterpret_cast<const char*>(&f_info), sizeof(File));
        std::cout << "Do you want to complete (1) or continue input (0)?" << std::endl;
        std::cin >> flag;
    }
}

void BinFile::delete_note() {
    std::map<int, File> map;
    make_map(map);
    bool flag = 0;
    int idx;
    while (!flag) {
        std::cout << "\nInput the index to delete: " << std::endl;
        std::cin >> idx;
        map.erase(idx);
        std::cout << "\nDo you want to complete (1) or continue input (0)?\n" << std::endl;
        std::cin >> flag;
    }
    std::ofstream bin_file(fname, std::ios::binary);
    if (!bin_file.is_open()) {
        std::cerr << "Error: cant open output file " << fname << std::endl;
        exit(EXIT_FAILURE);
    }
    for (const auto& f_info : map) {
        bin_file.write(reinterpret_cast<const char*>(&f_info.second), sizeof(File));
    }
}

void BinFile::replace_note() {
    std::map<int, File> map;
    make_map(map);
    int idx_to_replace;
    File file_info;
    std::cout << "Enter the index of the note you would like to replace:\n";
    std::cin >> idx_to_replace;
    std::cout << "Enter what you would like to replace the note with:\n";
    std::cin >> file_info;
    std::ofstream bin_file(fname, std::ios::binary);
    if (!bin_file.is_open()) {
        std::cerr << "Error: cant open output file " << fname << std::endl;
        exit(EXIT_FAILURE);
    }
    map[idx_to_replace] = file_info;
    for (const auto& car : map) {
        bin_file.write(reinterpret_cast<const char*>(&car.second), sizeof(File));
    }
}

void BinFile::bin_to_txt() {
    std::map<int, File> map;
    make_map(map);
    std::string new_fname = fname + "_table.txt";
    std::ofstream file(new_fname);
    if (!file.is_open()) {
        std::cerr << "Error: cant open output file " << fname << std::endl;
        exit(EXIT_FAILURE);
    }

    file << std::setw(50) << " Files " << std::endl;
    file << "| ID | Catalog | File name | Extention |    Date & time  |  Attribute  | Deletion sign | Number of allocated sectors |" << std::endl;
    for (const auto& file_info : map) {
        file << "|" << std::setw(3) << file_info.first << " |"
            << std::setw(8) << file_info.second.catalog << " |"
            << std::setw(10) << file_info.second.fname << " |"
            << std::setw(10) << file_info.second.ext << " |";
        date_time d = file_info.second.dt;
        file << d.toString() << " |";
        file << std::setw(12) << file_info.second.attr << " |"
            << std::setw(14) << file_info.second.del_sign << " |"
            << std::setw(28) << file_info.second.al_sec << " |"
            << std::endl;
    }
}

std::vector<File> BinFile::search_by_catalog(const std::string& cat) {
    std::map<int, File> map;
    make_map(map);
    std::vector<File> res;
    for (const auto& file_info : map) {
        if (file_info.second.catalog == cat) {
            res.push_back(file_info.second);
        }
    }
    return res;
}

std::vector<File> BinFile::search_by_date(date_time d) {
    std::map<int, File> map;
    make_map(map);
    std::vector<File> res;
    for (const auto& file_info : map) {
        if ((file_info.second.dt.year == d.year) && 
            (file_info.second.dt.month == d.month) &&
            (file_info.second.dt.day == d.day)) {
            res.push_back(file_info.second);
        }
    }
    return res;
}

std::vector<File> BinFile::search_by_del_sign(bool b) {
    std::map<int, File> map;
    make_map(map);
    std::vector<File> res;
    for (const auto& file_info : map) {
        if (file_info.second.del_sign == b) {
            res.push_back(file_info.second);
        }
    }
    return res;
}